@@ -36,19 +36,19 @@ gen_filename(std::string &fileName, bool swapObase, const std::string &obase, co
 }
 
 static int
-split_code(bool swapObase, const std::string &fileSuffix, std::string &fileName, int vlistID1, const VarList &varList1,
+split_code(bool swapObase, const std::string &fileSuffix, const std::string &fileName, int vlistID1, const VarList &varList,
            std::vector<CdoStreamID> &streamIDs, std::vector<int> &vlistIDs)
 {
-  auto nvars = vlistNvars(vlistID1);
-  std::vector<int> codes(nvars);
+  auto numVars = varList.numVars();
+  std::vector<int> codes(numVars);
 
   int nsplit = 0;
-  for (int varID = 0; varID < nvars; ++varID)
+  for (int varID = 0; varID < numVars; ++varID)
     {
-      const auto &var = varList1[varID];
+      const auto &var = varList.vars[varID];
       int index;
       for (index = 0; index < varID; ++index)
-        if (var.code == varList1[index].code) break;
+        if (var.code == varList.vars[index].code) break;
 
       if (index == varID) codes[nsplit++] = var.code;
     }
@@ -59,9 +59,9 @@ split_code(bool swapObase, const std::string &fileSuffix, std::string &fileName,
   for (int index = 0; index < nsplit; ++index)
     {
       vlistClearFlag(vlistID1);
-      for (int varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < numVars; ++varID)
         {
-          const auto &var = varList1[varID];
+          const auto &var = varList.vars[varID];
           if (codes[index] == var.code)
             {
               for (int levelID = 0; levelID < var.nlevels; ++levelID)
@@ -87,20 +87,20 @@ split_code(bool swapObase, const std::string &fileSuffix, std::string &fileName,
 }
 
 static int
-split_param(bool swapObase, const std::string &fileSuffix, std::string &fileName, int vlistID1, const VarList &varList1,
+split_param(bool swapObase, const std::string &fileSuffix, const std::string &fileName, int vlistID1, const VarList &varList,
             std::vector<CdoStreamID> &streamIDs, std::vector<int> &vlistIDs)
 {
   char paramstr[32];
-  auto nvars = vlistNvars(vlistID1);
-  std::vector<int> params(nvars);
+  auto numVars = varList.numVars();
+  std::vector<int> params(numVars);
 
   int nsplit = 0;
-  for (int varID = 0; varID < nvars; ++varID)
+  for (int varID = 0; varID < numVars; ++varID)
     {
-      const auto &var = varList1[varID];
+      const auto &var = varList.vars[varID];
       int index;
       for (index = 0; index < varID; ++index)
-        if (var.param == varList1[index].param) break;
+        if (var.param == varList.vars[index].param) break;
 
       if (index == varID) params[nsplit++] = var.param;
     }
@@ -111,9 +111,9 @@ split_param(bool swapObase, const std::string &fileSuffix, std::string &fileName
   for (int index = 0; index < nsplit; ++index)
     {
       vlistClearFlag(vlistID1);
-      for (int varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < numVars; ++varID)
         {
-          const auto &var = varList1[varID];
+          const auto &var = varList.vars[varID];
           if (params[index] == var.param)
             {
               for (int levelID = 0; levelID < var.nlevels; ++levelID)
@@ -140,11 +140,11 @@ split_param(bool swapObase, const std::string &fileSuffix, std::string &fileName
 }
 
 static int
-split_name(bool swapObase, const std::string &fileSuffix, std::string &fileName, int vlistID1, const VarList &varList1,
+split_name(bool swapObase, const std::string &fileSuffix, const std::string &fileName, int vlistID1, const VarList &varList,
            std::vector<CdoStreamID> &streamIDs, std::vector<int> &vlistIDs)
 {
-  auto nvars = vlistNvars(vlistID1);
-  auto nsplit = nvars;
+  auto numVars = varList.numVars();
+  auto nsplit = numVars;
 
   vlistIDs.resize(nsplit);
   streamIDs.resize(nsplit);
@@ -153,7 +153,7 @@ split_name(bool swapObase, const std::string &fileSuffix, std::string &fileName,
     {
       vlistClearFlag(vlistID1);
       int varID = index;
-      const auto &var = varList1[varID];
+      const auto &var = varList.vars[varID];
       for (int levelID = 0; levelID < var.nlevels; ++levelID)
         {
           vlistDefIndex(vlistID1, varID, levelID, index);
@@ -164,7 +164,7 @@ split_name(bool swapObase, const std::string &fileSuffix, std::string &fileName,
       cdo_vlist_copy_flag(vlistID2, vlistID1);
       vlistIDs[index] = vlistID2;
 
-      std::string formatted =  fileName + var.name;
+      std::string formatted = fileName + var.name;
       gen_filename(formatted, swapObase, cdo_get_obase(), fileSuffix);
 
       streamIDs[index] = cdo_open_write(formatted);
@@ -174,11 +174,11 @@ split_name(bool swapObase, const std::string &fileSuffix, std::string &fileName,
 }
 
 static int
-split_level(bool swapObase, const std::string &fileSuffix, std::string &fileName, int vlistID1, const VarList &varList1,
+split_level(bool swapObase, const std::string &fileSuffix, const std::string &fileName, int vlistID1, const VarList &varList,
             std::vector<CdoStreamID> &streamIDs, std::vector<int> &vlistIDs)
 {
-  auto nvars = vlistNvars(vlistID1);
-  auto nzaxis = vlistNzaxis(vlistID1);
+  auto numVars = varList.numVars();
+  auto nzaxis = vlistNumZaxis(vlistID1);
   Varray<double> ftmp(999, 0.0);
 
   int nsplit = 0;
@@ -204,9 +204,9 @@ split_level(bool swapObase, const std::string &fileSuffix, std::string &fileName
   for (int index = 0; index < nsplit; ++index)
     {
       vlistClearFlag(vlistID1);
-      for (int varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < numVars; ++varID)
         {
-          const auto &var = varList1[varID];
+          const auto &var = varList.vars[varID];
           for (int levelID = 0; levelID < var.nlevels; ++levelID)
             {
               auto level = cdo_zaxis_inq_level(var.zaxisID, levelID);
@@ -232,11 +232,11 @@ split_level(bool swapObase, const std::string &fileSuffix, std::string &fileName
 }
 
 static int
-split_grid(bool swapObase, const std::string &fileSuffix, std::string &fileName, int vlistID1, const VarList &varList1,
+split_grid(bool swapObase, const std::string &fileSuffix, const std::string &fileName, int vlistID1, const VarList &varList,
            std::vector<CdoStreamID> &streamIDs, std::vector<int> &vlistIDs)
 {
-  auto nsplit = vlistNgrids(vlistID1);
-  auto nvars = vlistNvars(vlistID1);
+  auto nsplit = vlistNumGrids(vlistID1);
+  auto numVars = varList.numVars();
 
   vlistIDs.resize(nsplit);
   streamIDs.resize(nsplit);
@@ -246,9 +246,9 @@ split_grid(bool swapObase, const std::string &fileSuffix, std::string &fileName,
   for (int index = 0; index < nsplit; ++index)
     {
       vlistClearFlag(vlistID1);
-      for (int varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < numVars; ++varID)
         {
-          const auto &var = varList1[varID];
+          const auto &var = varList.vars[varID];
           if (gridIDs[index] == var.gridID)
             {
               for (int levelID = 0; levelID < var.nlevels; ++levelID)
@@ -273,11 +273,11 @@ split_grid(bool swapObase, const std::string &fileSuffix, std::string &fileName,
 }
 
 static int
-split_zaxis(bool swapObase, const std::string &fileSuffix, std::string &fileName, int vlistID1, const VarList &varList1,
+split_zaxis(bool swapObase, const std::string &fileSuffix, const std::string &fileName, int vlistID1, const VarList &varList,
             std::vector<CdoStreamID> &streamIDs, std::vector<int> &vlistIDs)
 {
-  auto nsplit = vlistNzaxis(vlistID1);
-  auto nvars = vlistNvars(vlistID1);
+  auto nsplit = vlistNumZaxis(vlistID1);
+  auto numVars = varList.numVars();
 
   vlistIDs.resize(nsplit);
   streamIDs.resize(nsplit);
@@ -287,9 +287,9 @@ split_zaxis(bool swapObase, const std::string &fileSuffix, std::string &fileName
   for (int index = 0; index < nsplit; ++index)
     {
       vlistClearFlag(vlistID1);
-      for (int varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < numVars; ++varID)
         {
-          const auto &var = varList1[varID];
+          const auto &var = varList.vars[varID];
           if (zaxisIDs[index] == var.zaxisID)
             {
               for (int levelID = 0; levelID < var.nlevels; ++levelID)
@@ -304,7 +304,6 @@ split_zaxis(bool swapObase, const std::string &fileSuffix, std::string &fileName
       cdo_vlist_copy_flag(vlistID2, vlistID1);
       vlistIDs[index] = vlistID2;
 
-
       std::string formatted = fileName + string_format("%02d", vlistZaxisIndex(vlistID1, zaxisIDs[index]) + 1);
       gen_filename(formatted, swapObase, cdo_get_obase(), fileSuffix);
 
@@ -315,14 +314,14 @@ split_zaxis(bool swapObase, const std::string &fileSuffix, std::string &fileName
 }
 
 static int
-split_tabnum(bool swapObase, const std::string &fileSuffix, std::string &fileName, int vlistID1, const VarList &varList1,
+split_tabnum(bool swapObase, const std::string &fileSuffix, const std::string &fileName, int vlistID1, const VarList &varList,
              std::vector<CdoStreamID> &streamIDs, std::vector<int> &vlistIDs)
 {
-  auto nvars = vlistNvars(vlistID1);
-  std::vector<int> tabnums(nvars);
+  auto numVars = varList.numVars();
+  std::vector<int> tabnums(numVars);
 
   int nsplit = 0;
-  for (int varID = 0; varID < nvars; ++varID)
+  for (int varID = 0; varID < numVars; ++varID)
     {
       auto tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
       int index;
@@ -338,12 +337,12 @@ split_tabnum(bool swapObase, const std::string &fileSuffix, std::string &fileNam
   for (int index = 0; index < nsplit; ++index)
     {
       vlistClearFlag(vlistID1);
-      for (int varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < numVars; ++varID)
         {
           auto tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
           if (tabnums[index] == tabnum)
             {
-              const auto &var = varList1[varID];
+              const auto &var = varList.vars[varID];
               for (int levelID = 0; levelID < var.nlevels; ++levelID)
                 {
                   vlistDefIndex(vlistID1, varID, levelID, index);
@@ -365,15 +364,27 @@ split_tabnum(bool swapObase, const std::string &fileSuffix, std::string &fileNam
   return nsplit;
 }
 
-class ModuleSplit
+class Split : public Process
 {
-  int SPLITCODE;
-  int SPLITPARAM;
-  int SPLITNAME;
-  int SPLITLEVEL;
-  int SPLITGRID;
-  int SPLITZAXIS;
-  int SPLITTABNUM;
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Split",
+    .operators = { { "splitcode", SplitHelp },
+                   { "splitparam", SplitHelp },
+                   { "splitname", SplitHelp },
+                   { "splitlevel", SplitHelp },
+                   { "splitgrid", SplitHelp },
+                   { "splitzaxis", SplitHelp },
+                   { "splittabnum", SplitHelp } },
+    .aliases = { { "splitvar", "splitname" } },
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_BOTH,  // Allowed number type
+    .constraints = { 1, OBASE, OnlyFirst },
+  };
+  inline static RegisterEntry<Split> registration = RegisterEntry<Split>(module);
+
+  int SPLITCODE, SPLITPARAM, SPLITNAME, SPLITLEVEL, SPLITGRID, SPLITZAXIS, SPLITTABNUM;
 
   std::vector<int> vlistIDs;
   std::vector<CdoStreamID> streamIDs;
@@ -387,24 +398,17 @@ class ModuleSplit
 
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
-    if (process_self().m_ID != 0) cdo_abort("This operator can't be combined with other operators!");
-
     dataIsUnchanged = data_is_unchanged();
 
-    // clang-format off
-    SPLITCODE   = cdo_operator_add("splitcode",   0, 0, nullptr);
-    SPLITPARAM  = cdo_operator_add("splitparam",  0, 0, nullptr);
-    SPLITNAME   = cdo_operator_add("splitname",   0, 0, nullptr);
-    SPLITLEVEL  = cdo_operator_add("splitlevel",  0, 0, nullptr);
-    SPLITGRID   = cdo_operator_add("splitgrid",   0, 0, nullptr);
-    SPLITZAXIS  = cdo_operator_add("splitzaxis",  0, 0, nullptr);
-    SPLITTABNUM = cdo_operator_add("splittabnum", 0, 0, nullptr);
-    // clang-format on
+    SPLITCODE = module.get_id("splitcode");
+    SPLITPARAM = module.get_id("splitparam");
+    SPLITNAME = module.get_id("splitname");
+    SPLITLEVEL = module.get_id("splitlevel");
+    SPLITGRID = module.get_id("splitgrid");
+    SPLITZAXIS = module.get_id("splitzaxis");
+    SPLITTABNUM = module.get_id("splittabnum");
 
     auto operatorID = cdo_operator_id();
 
@@ -423,7 +427,7 @@ public:
     streamID1 = cdo_open_read(0);
     vlistID1 = cdo_stream_inq_vlist(streamID1);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
     std::string fileName;
     if (!swapObase) fileName = cdo_get_obase();
@@ -468,21 +472,20 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     Field field;
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         for (int index = 0; index < nsplit; ++index) cdo_def_timestep(streamIDs[index], tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
             auto index = vlistInqIndex(vlistID1, varID, levelID);
             auto vlistID2 = vlistIDs[index];
@@ -490,14 +493,14 @@ public:
             auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
             // printf("%d %d %d %d %d %d\n", index, vlistID2, varID, levelID, varID2, levelID2);
 
-            cdo_def_record(streamIDs[index], varID2, levelID2);
-            if (dataIsUnchanged) { cdo_copy_record(streamIDs[index], streamID1); }
+            cdo_def_field(streamIDs[index], varID2, levelID2);
+            if (dataIsUnchanged) { cdo_copy_field(streamIDs[index], streamID1); }
             else
               {
-                const auto &var = varList1[varID];
+                const auto &var = varList1.vars[varID];
                 field.init(var);
-                cdo_read_record(streamID1, field);
-                cdo_write_record(streamIDs[index], field);
+                cdo_read_field(streamID1, field);
+                cdo_write_field(streamIDs[index], field);
               }
           }
 
@@ -506,24 +509,11 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
 
-    for (auto &streamID : streamIDs) cdo_stream_close(streamID);
-    for (auto &id : vlistIDs) vlistDestroy(id);
-
-    cdo_finish();
+    for (const auto &streamID : streamIDs) cdo_stream_close(streamID);
+    for (const auto &id : vlistIDs) vlistDestroy(id);
   }
 };
-
-void *
-Split(void *process)
-{
-  ModuleSplit split;
-  split.init(process);
-  split.run();
-  split.close();
-
-  return nullptr;
-}
